'------------------------------------------------------------------------------
' File:        EXAMPLE.PRG
' Purpose:     EtherCAT setup example
' Version:     1.00
' Author:      Nir Geller
' Description: This script demonstrates how to setup the EtherCAT module, which function/subroutines to invoke,
'              and in what order. 
'
' History:     20.10.2013  V1.00
'------------------------------------------------------------------------------

' module global "constants"

' module global variables

common shared slave0_vendor as long = 0x000002e1  ' Slave 0 vendor ID
common shared slave0_product as long = 0x00       ' Slave 0 product ID

common shared slave1_vendor as long = 0x00000017  ' Slave 1 vendor ID
common shared slave1_product as long = 0x00000004    ' Slave 1 product ID

common shared Pos_Units[32] as long

common shared numOfSlaves as long = 0



Program 

	dim user_cycletime as long = 0
	dim counter as long = 0
	dim retVal as long = 0
	dim i as long = 0

	call ec_stopmaster

	sleep 500


	while (numOfSlaves = 0) and (counter < 10)
		numOfSlaves = EC_NRSLAVES()
		counter = counter + 1
		sleep 100
	end while

	if numOfSlaves = 0 then
		throw EC_MASTER_NO_SLAVES_FOUND
	end if


	user_cycletime = 1000

	EC_ADDRESS_BY_ALIAS = 0 ' Set to zero -> alias addressing is position based


	Try
		call EC_ETHERCAT_INIT ' Initialize EtherCAT according to a certain field bus, and type of slave addressing
	Catch 20037
		print "A motion drive is enabled. Disable all motion drives and rerun EC_SETUP.PRG"
		KTas$ programname
	End Try


	' Set CDHD Drives with PDOS that allow sync position Mode
	for i = 1 to numOfSlaves
		if (ETSLAVES[i]->et_slavetype = ECAT_MOTIONSLAVE) then
			call EC_INIT_MOTION_SLAVE(i, EC_BUSID, EC_ADDRESS_BY_ALIAS , 8)
		end if
	next

	EC_INIT(EC_ADDRESS_BY_ALIAS)

	call EC_ASSIGN_PDOS_AUTO


	'*******************************************************************************************************************
	' If an IO module is a part of the topology it needs to be attended specifically.
	' Break the following For Loop and make sure that in the correct position the subroutine EC_USE_SLAVE is invoked
	' with the argument "ECAT_IOMODULE_SLAVE". 
	'
	'                                DO NOT ASSIGN THE IO MODULE'S ADDRESS TO AN AXIS
	'
	' Instead, for example, invoke the following subroutine that can be found in EC_IOMOD.LIB:
	'
	' public sub EC_INSTALL_ECX_DIO8(byval slaveAddr as long)
	'
	'*******************************************************************************************************************
	for i = 1 to numOfSlaves
		' for CDHD drives : Choose which slaves to use according to their address
		' "ECAT_AXIS_SLAVE"  indicates it is a drive slave, and not a IO module
		' "ECAT_IOMODULE_SLAVE"  indicates it is an IO module
		call EC_USE_SLAVE(i, slave0_vendor, slave0_product, "ECAT_AXIS_SLAVE")

	next

	print "Found ";numOfSlaves;" slaves"

	' After telling the master with which slaves we would like to work, we create the PDO map
	call EC_CREATE_PDOS_MAP

	call EC_SET_CYCLETIME(user_cycletime)  ' Set EtherCAT cycle time

	call EC_STARTMASTER ' sync clocks between the master and the slaves, and sets master's opmode to operational

	' User setup of slaves
	for i = 1 to numOfSlaves 
		
		' Clear faults in the drives. If clearing fault fails, an error is thrown and EC Setup stops
		call EC_CLEAR_FAULTS(i)
		
		sleep 1000
		
		' Assign slave address to each axis
		Axes[i].dadd = i
		
		' Set drive to PREOP
		call EC_Slave_NMT_StateSet(i, EC_STATE_PREOP)
		
		'  Micro Interpolation
		call EC_SDO_WRITE(Axes[i].dadd, 0x60C0, 0, 16, 2)
		
		' define pos encode resolution
		' read mencres - native encoder resolution
		Try
			Pos_Units[i] = EC_SDO_READ(Axes[i].dadd, 0x20f1,0)
		Catch Else
			Pos_Units[i] = EC_SDO_READ(Axes[i].dadd, POS_ENCODE_RES_INDEX_0x608F,POS_ENCODE_RES_SUBINDEX_0x1)
		End Try

		if  Pos_Units[i] <= 256 then 
			Pos_Units[i] = Pos_Units[i] * 2^16 ' sine encoder
		else
			Pos_Units[i] = Pos_Units[i] * 4
		end if

		' drive will get/send data in encoder counts
		call EC_SDO_WRITE(Axes[i].dadd, POS_SHAFT_RES_0x6092, POS_SHAFT_RES_SUBINDEX_0x1, 32, Pos_Units[i])  
		call EC_SDO_WRITE(Axes[i].dadd, POS_SHAFT_RES_0x6092, POS_SHAFT_RES_SUBINDEX_0x2, 32, 1)
		call EC_SDO_WRITE(Axes[i].dadd, 0x6091, 1, 32, 1)  
		call EC_SDO_WRITE(Axes[i].dadd, 0x6091, 2, 32, 1)

		' Set drive to SAFEOP
		call EC_Slave_NMT_StateSet(i, EC_STATE_SAFEOP)
		' Set drive to OP
		call EC_Slave_NMT_StateSet(i, EC_STATE_OP)

		
		' Set CDHD drives to sync position mode
		if EC_IS_PDO(Axes[i].dadd, 0x6060, 0) then
			call EC_PDO_WRITE(Axes[i].dadd, Modes_Of_Operation_6060h, 0, 8)
		else
			call EC_SDO_WRITE(Axes[i].dadd, Modes_Of_Operation_6060h, 0, 8, 8)
		end if

		
		print "Slave ";i;": encoder resolution is set"
		
	next
	
	
	' Associate drives' digital outputs with system digital outputs
	' COMPLIANT SPECIFICALLY WITH CDHD !!
	call EC_USE_STX_CDHD

	for i = 1 to numOfSlaves
		call EC_SET_CDHD_INPUTS(i, i*100, 11)
		call EC_SET_CDHD_OUTPUTS(i, i*100, 4)
		printu "Drive  # : sys.din. # to sys.din. #" ;i, i*100 , i*100+10
		printu "Drive  # : sys.dout. # to sys.dout. #";i, i*100 , i*100+3
	next
	
	print "Ethercat Setup Done!"


End Program




