'---------------------------------------------------------------------------------------------
' File:        MANPDOMP.PRG
' Purpose:     Manual PDO Mapping example
' Version:     1.00
' Author:      Nir Geller
' Description: This script demonstrates how to manually map specific PDO's of the CDHD drive
'
' History:     24.4.2014  V1.00
'---------------------------------------------------------------------------------------------


'*********************************************************************************************
'
' PDO mapping is done when the drive's EtherCAT operational mode is PREOP.
'
' First the user must handle the sync manager. In CDHD, object 0x1C12 holds Rx PDOs and object
' 0x1C13 holds Tx PDOs. In these objects sub index 0 holds the number of the PDO's and sub indices
' 1 - 4 hold their addresses. 0x1600-3 are RxPDOs and they are set in sync manager 0x1C12. 0x1A00-3
' are TxPDOs and they are set in sync manager 0x1C13. If the user needs only Rx PDOs 0x1600 and 0x1601,
' then object 0x1C12 sub index 0 must be set to 2, sub index 1 to 0x1600 and sub index 2 to 0x1601.
'
' In order to map CDHD PDO entries one must write via SDO to objects 0x1600-3(Rx) and 0x1A00-3(Tx).
' In these objects sub index 0 holds the number of entries, meaning the number of mapped PDO's.
' Sub indices 1 to 6 hold 32 bits values that define the objects PDO mapping in the following manner:
' 0xAAAABBCC, where 0xAAAA is the object's address, BB the object's sub index, CC the object's size
' in hexadecimal format (08 - 8 bits, 10 - 16 bits, 20 - 32 bits).
'
' The correct way to do so is first write 0 to number of entries (sub index 0), write the desired values
' to sub indices 1 to 6, and then write the new number of mapped PDO's back to sub index 0.
' After this is done, invoke the function EC_RESCAN_SLAVES.
'
' In the linux terminal the user can type 'ethercat pdos' in order to make sure the new PDO mapping
' is acknowledged by the ECat master.
'
' The system is now ready to turn into OPMODE.
'
'*********************************************************************************************


' PDO mapping objects
common shared EC_RX_SYNC_MANAGER_1C12 as const long = 0x1C12
common shared EC_TX_SYNC_MANAGER_1C13 as const long = 0x1C13

common shared EC_RxPDO_1600 as const long = 0x1600
common shared EC_RxPDO_1601 as const long = 0x1601
common shared EC_RxPDO_1602 as const long = 0x1602
common shared EC_RxPDO_1603 as const long = 0x1603

common shared EC_TxPDO_1A00 as const long = 0x1A00
common shared EC_TxPDO_1A01 as const long = 0x1A01
common shared EC_TxPDO_1A02 as const long = 0x1A02
common shared EC_TxPDO_1A03 as const long = 0x1A03

common shared num_of_slaves as long = 0

Program 

	dim counter as long = 0
	dim retVal as long = 0

	call ec_stopmaster
	num_of_slaves = 0
	retVal = EC_RESCAN_SLAVES

	sleep 500

	while (num_of_slaves = 0) and (counter < 10)
		num_of_slaves = EC_Num_Of_Slaves
		counter = counter + 1
		sleep 100
	end while

	if num_of_slaves = 0 then
		throw EC_MASTER_NO_SLAVES_FOUND
	end if
	
	?EC_CDHD_MAP_PDOS
	
	
End Program


'****************************************************************************
' Function Name: EC_CDHD_MAP_PDOS
' Description: This function determines the PDO mapping of the CDHD drive
' Called From:
' Author: Nir Geller
' Input Parameters: None
' Output Parameters: None
' Return Value: 0 if successful, or -1 if failed
' Algorithm: 
' Global Variables Used:
' Revisions:
'****************************************************************************
Function EC_CDHD_MAP_PDOS as long


'****************************************************************************
'
'		THIS FUNCTION MAPS THE MINIMUM REQUIRED PDOS ONLY IN THE FIRST DRIVE
'		- CONTROL WORD
'		- STATUS WORD
'		- PCMD
'		- VCMD
'		- PFB
'		- Digital Outputs
'		- Digital Inputs
'
'****************************************************************************

	dim i as long = 0

	'for i = 1 to num_of_slaves
	i = 1

		'********************************************************************
		' 						Set Rx PDOs
		'********************************************************************

		' Zero number of entries in sync manager 2
		call EC_SDO_WRITE(i, EC_RX_SYNC_MANAGER_1C12, 0, 8, 0)

		' Set Rx PDO Entries object
		call EC_SDO_WRITE(i, EC_RX_SYNC_MANAGER_1C12, 1, 16, 0x1600)

		' Set number of entries
		call EC_SDO_WRITE(i, EC_RX_SYNC_MANAGER_1C12, 0, 8, 1)


		' Zero number of entries
		call EC_SDO_WRITE(i, EC_RxPDO_1600, 0, 8, 0)

		' Set ControlWord
		call EC_SDO_WRITE(i, EC_RxPDO_1600, 1, 32, 0x60400010)

		' Set Position Command
		call EC_SDO_WRITE(i, EC_RxPDO_1600, 2, 32, 0x607A0020)

		' Set Velocity Command
		call EC_SDO_WRITE(i, EC_RxPDO_1600, 3, 32, 0x60810020)
		
		' Set Digital Outputs
		call EC_SDO_WRITE(i, EC_RxPDO_1600, 4, 32, 0x60FE0120)

		' Set number of entries
		call EC_SDO_WRITE(i, EC_RxPDO_1600, 0, 8, 4)

		' ADD HERE ADDITIONAL PDOS TO OBJECTS 1600-3
		' **
		' **
		' **



		'********************************************************************
		' 						Set Tx PDOs
		'********************************************************************

		' Zero number of entries in sync manager 3
		call EC_SDO_WRITE(i, EC_TX_SYNC_MANAGER_1C13, 0, 8, 0)

		' Set Tx PDO Entries object
		call EC_SDO_WRITE(i, EC_TX_SYNC_MANAGER_1C13, 1, 16, 0x1A00)

		' Set number of entries
		call EC_SDO_WRITE(i, EC_TX_SYNC_MANAGER_1C13, 0, 8, 1)


		' Zero number of entries
		call EC_SDO_WRITE(i, EC_TxPDO_1A00, 0, 8, 0)

		' Set StatusWord
		call EC_SDO_WRITE(i, EC_TxPDO_1A00, 1, 32, 0x60410010)

		' Set Position Feedback
		call EC_SDO_WRITE(i, EC_TxPDO_1A00, 2, 32, 0x60640020)

		' Set Digital Inputs
		call EC_SDO_WRITE(i, EC_TxPDO_1A00, 3, 32, 0x60FD0020)

		' Set number of entries
		call EC_SDO_WRITE(i, EC_TxPDO_1A00, 0, 8, 3)

		' ADD HERE ADDITIONAL PDOS TO OBJECTS 1A00-3
		' **
		' **
		' **

	'next

	EC_CDHD_MAP_PDOS = EC_RESCAN_SLAVES

End Function


